<?php
// $Id: luceneapi_facet.block.inc,v 1.1.2.1 2009/08/18 03:27:50 cpliakas Exp $

/**
 * @file
 * Renders the Search Lucene Facets facet block.
 *
 * @ingroup luceneapi
 */

/**
 * Displays the actual number of results that will be displayed when clicking
 * on the facet.
 *
 * @param $facets
 *   An array of enabled facets.
 * @param $index
 *   A LuceneAPI_Search_Lucene_Proxy object, such as the one returned by the
 *   luceneapi_index_open() function.
 * @param $query
 *   A Zend_Search_Lucene_Search_Query object modeling the query.
 * @return
 *   An array of items.
 * @throws LuceneAPI_Exception
 */
function _luceneapi_facet_count_block_get(LuceneAPI_Search_Lucene_Proxy $index, Zend_Search_Lucene_Search_Query $query, array $facets) {
  $module = $index->getModule();
  foreach ($facets as $name => $facet) {

    // initializes array containing facet information
    $item = array(
      'name' => $name,
      'title' => $facet['title'],
      'field' => $facet['field'],
      'element' => $facet['element'],
      'selected' => array(),
    );

    // gets list of selected values from ones passed through the query string
    $passed = luceneapi_facet_value_get($facet['element'], '');
    $item['selected'] = array();
    if (!empty($passed)) {
      if (is_array($passed)) {
        foreach ($passed as $cur_passed) {
          if ($cur_passed && isset($facet['values'][$cur_passed])) {
            $item['selected'][$cur_passed] = $cur_passed;
          }
        }
      }
      else {
        $item['selected'] = array($passed => $passed);
      }
    }

    // builds array of facet items
    $item['items'] = array();
    foreach ($facet['values'] as $option => $text) {
      $qstring = $_GET;
      if (!isset($item['selected'][$option])) {
        $theme_function = 'luceneapi_facet_link';
        $qstring[$facet['element']][$option] = $option;
      }
      else {
        $theme_function = 'luceneapi_facet_link_selected';
        unset($qstring[$facet['element']][$option]);
      }
      $options = array(
        'query' => drupal_query_string_encode($qstring, array('q', 'page'))
      );
      $item['items'][$option] = theme(
        $theme_function, t($text), $_GET['q'], $options
      );
    }
    $items[] = $item;
  }

  // re-executes search query, calculates counts for selected facets
  $hits = luceneapi_find($index, $query, array(), $positive_keys, TRUE);
  $max = count($hits) + 1;

  // adds item count to array
  foreach ($hits as $hit) {
    foreach ($items as $key => $item) {
      if (!isset($items[$key]['count'])) {
        $items[$key]['count'] = array();
      }

      // normalizes values to an array, some fields like taxonomy terms are
      // multiple values separated by spaces, whereas other fields such as
      // the username may have spaces in the values.
      $value = $hit->$item['field'];
      if (isset($items['key']['values'][$value])) {
        $values = array($value);
      }
      else {
        $values = explode(' ', $value);
      }

      // counts all items
      foreach ($values as $value) {
        if (!isset($items[$key]['count'][$value])) {
          $items[$key]['count'][$value] = 0;
        }
        $items[$key]['count'][$value]++;
      }
    }
  }

  // adds count to items, strips ones that do not exist
  foreach ($items as $key => $item) {
    foreach ($item['items'] as $option => $value) {
      if (!empty($items[$key]['count'][$option])) {
        if (isset($item['selected'][$option])) {
          $items[$key]['count'][$option] = $max;
        }
        else {
          if (!empty($items[$key]['count'][$option])) {
            $items[$key]['items'][$option] .= theme(
              'luceneapi_facet_count',
              $items[$key]['count'][$option]
            );
          }
        }
      }
      else {
        unset($items[$key]['items'][$option]);
      }
    }
    if (empty($items[$key]['items']) && !isset($items[$key]['items_searched'])) {
      unset($items[$key]);
    }
  }

  // sorts by score
  foreach ($items as $key => $item) {

    // sorts by highest count
    arsort($items[$key]['count']);
    $total = count($items[$key]['count']);

    // orders by how many are left
    $tmp_items = $items[$key]['items'];
    $items[$key]['items'] = array_intersect_key($items[$key]['count'], $tmp_items);

    foreach ($items[$key]['items'] as $option => $count) {
      $items[$key]['items'][$option] = $tmp_items[$option];
    }
  }

  return $items;
}
